/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_EVENT_DISPATCHER_H__
#define __TR_EVENT_DISPATCHER_H__

#include <time.h>
#include <stdint.h>

#include "trbase.h"
#include "trdata.h"

#include "tr/event.h"

TR_CLASS(TR_EventDispatcher);

typedef enum TR_EventDone_e {
	TR_EVENT_DONE = 0,
	TR_EVENT_PENDING
} TR_EventDone;

#include "tr/event_handler.h"

typedef enum TR_e_EventDispatcherMode {
	TR_EVD_SERVER = 0,
	TR_EVD_CLIENT
} TR_EventDispatcherMode;

TR_CLASS(TR_EventDispatcher) {
	TR_EXTENDS(TR_EventSubject);

	TR_Queue               events;
	TR_Hash                handler;
	TR_EventHandler        default_handler;
	int                    running;
	TR_Timer               heartbeat;
	unsigned long          n_beats;
	TR_EventDispatcherMode mode;
};
TR_INSTANCE_INIT(TR_EventDispatcher);
TR_CLASSVARS_DECL(TR_EventDispatcher) {
	TR_CV_EXTENDS(TR_EventSubject);
};

#define TR_DISPATCHER_EVENT_HEARTBEAT	0
#define TR_DISPATCHER_EVENT_USER_WAIT	1
#define TR_DISPATCHER_EVENT_DATA_WAIT	2
#define TR_DISPATCHER_EVENT_SHUTDOWN	3
#define TR_DISPATCHER_EVENT_MAX			((size_t)TR_DISPATCHER_EVENT_SHUTDOWN)

#define TR_DISPATCHER_MODE_TO_EVENTID(mode) \
	((mode) == TR_EVD_CLIENT                \
	 ? TR_DISPATCHER_EVENT_USER_WAIT        \
	 : TR_DISPATCHER_EVENT_DATA_WAIT)

void TR_eventDispatcherRegisterHandler(TR_EventDispatcher, TR_EventHandler);
unsigned long
TR_eventDispatcherGetDataWaitTime(TR_EventDispatcher);
void TR_eventDispatcherStart(TR_EventDispatcher);
void TR_eventDispatcherShutdown(TR_EventDispatcher);

#define TR_eventDispatcherEnqueueEvent(disp,ev)	\
	(TR_queuePut((disp)->events, (ev)))
#define TR_eventDispatcherStop(disp) \
	(((TR_EventDispatcher)disp)->running = 0)
#define TR_eventDispatcherSetHeartbeat(disp, beat) \
	(TR_timerSetMil((disp)->heartbeat, (beat)))
#define TR_eventDispatcherGetBeatTime(disp) \
	(TR_timerGet((disp)->heartbeat, &((disp)->n_beats)))

#endif // __TR_EVENT_DISPATCHER_H__

// vim: set ts=4 sw=4:

