/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_EVENT_SUBJECT_H__
#define __TR_EVENT_SUBJECT_H__

#include <sys/types.h>
#include <stdint.h>

#include "trbase.h"

TR_CLASS(TR_EventSubject) {
	int    fin;
	size_t emitted;
};
TR_INSTANCE_INIT(TR_EventSubject);
TR_CLASSVARS_DECL(TR_EventSubject) {
	const char ** eventStrings;
	size_t        nevents;
	intptr_t    * events;
};

#include "tr/event.h"

#define TR_EVENT_CREATE(cls, event)                         \
	TR_CLASSVARS(TR_EventSubject, (cls))->events[(event)] = \
		(((intptr_t)(cls)) << 8 | (event))
#define TR_EVENT_DESTROY(cls, event) \
	TR_CLASSVARS(TR_EventSubject, (cls))->events[(event)] = -1

#define TR_eventSubjectId(cname, idx) \
	TR__eventSubjectId(TR_CLASS_BY_NAME(cname), idx)
#define TR_eventSubjectGetId(obj, idx) \
	TR__eventSubjectId(TR_GET_CLASS(obj), idx)

#define TR_eventSubjectHasUnhandledEvents(es) ((es)->emitted > 0)

intptr_t     TR__eventSubjectId(TR_class_ptr, size_t);
TR_Event     TR_eventSubjectEmit(TR_EventSubject, int, void *);

#define TR_eventSubjectFinalize(es) ((es)->fin = TRUE)

#endif // __TR_EVENT_SUBJECT_H__

// vim: set ts=4 sw=4:

