/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "trbase.h"
#include "trdata.h"
#include "trhash.h"

#include "tr/event.h"
#include "tr/event_subject.h"
#include "tr/event_dispatcher.h"

int ZERO = 0;

void
TR_eventDispatcherStart(TR_EventDispatcher this)
{
	this->running = 1;

	while (this->running || (! TR_queueEmpty(this->events))) {
		TR_Event        event;
		TR_List         handler_list;
		TR_HashValue    handler_list_hv;

		TR_eventDispatcherGetBeatTime(this);

		if (this->n_beats) {
			event = TR_eventSubjectEmit(
					(TR_EventSubject)this,
					TR_DISPATCHER_EVENT_HEARTBEAT,
					NULL);
		} else {
			event = TR_queueGet(this->events);

			if (! event) {
				int evtid = TR_EVD_CLIENT == this->mode
					? TR_DISPATCHER_EVENT_USER_WAIT
					: TR_DISPATCHER_EVENT_DATA_WAIT;

				event = TR_eventSubjectEmit((TR_EventSubject)this, evtid, NULL);
			}
		}

		handler_list_hv = TR_hashGetByVal(
				this->handler,
				TR_sdbm(
					(unsigned char *)&(event->id),
					sizeof(event->id)));

		handler_list = handler_list_hv
			? *(TR_List *)handler_list_hv->value
			: NULL;

		if (handler_list) {
			TR_EventDone done = TR_EVENT_PENDING;

			TR_iterableForeach(handler_list) {
				TR_EventHandler handler = TR_iterableCurrent(handler_list);
				TR_EventDone    this_done;

				this_done = TR_eventHandlerHandleEvent(handler, event);
				done = TR_EVENT_DONE == done ? done : this_done;
			}

			if (TR_EVENT_DONE == done) {
				TR_delete(event);
			} else {
				TR_eventDispatcherEnqueueEvent(this, event);
			}
		} else {
			TR_delete(event);
		}
	}
}

// vim: set ts=4 sw=4:
